/**
  *  \file client/dlg-build.h
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_DLG_BUILD_H
#define PCC_V2_CLIENT_DLG_BUILD_H

#include <vector>
#include "client/wcost.h"
#include "client/winfo.h"
#include "cpluslib/holder.h"
#include "game/spec.h"
#include "ui/listbox.h"
#include "ui/ui.h"
#include "ui/window.h"
#include "ui/widgets.h"
#include "game/actions/baseship.h"
#include "client/wcomplist.h"
#include "ui/widgets/imagewidget.h"

class GPlanet;

class WOrderDisplay : public UIWidget {
    TBuildOrder o;
 public:
    WOrderDisplay();
    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setBuildOrder(const TBuildOrder& o);
};

class WPartStorageDisplay : public UIWidget {
    int n;
 public:
    WPartStorageDisplay();
    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setCount(int n);
};

class WPartStorageGroup : public UIGroup {
    UIImageWidget image;
    WPartStorageDisplay psd;
    UISpacer      spacer;
    UIButton      button;
 public:
    WPartStorageGroup();
    ~WPartStorageGroup();

    void setCount(int n)
        { psd.setCount(n); }

    void setImage(Ptr<GfxPixmap> pix)
        { image.setImage(pix); }
};

class WPartCountInfo : public UIWidget {
    bool torps;
    int want, max;
 public:
    WPartCountInfo(bool torps);
    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setPartsMax(int n);
    void setPartsWanted(int n);
    int getPartsWanted() const
        { return want; }
    int getPartsMax() const
        { return max; }
};

class WPartCountGroup : public UIGroup {
    WPartCountInfo pci;
    UIGroup button_group;
    UIButton plus, minus;
 public:
    WPartCountGroup(bool torps);
    ~WPartCountGroup();

    bool handleCommand(const UICommand& cmd, UIBaseWidget& sender);

    void setPartsMax(int n);
    void setPartsWanted(int n)
        { pci.setPartsWanted(n); }
    int getPartsWanted() const
        { return pci.getPartsWanted(); }
    int getPartsMax() const
        { return pci.getPartsMax(); }
};

class WEngineInfoDiagram : public UIWidget {
    int engine;
 public:
    WEngineInfoDiagram();
    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setEngine(int n);
};

class WEngineStorageInfo : public UIWidget {
    int have, need;
 public:
    WEngineStorageInfo();
    void setEnginesAvailable(int n);
    void setEnginesNeeded(int n);
    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
};

class WShipBuildScreen : public UIWindow {
    WHullListbox      hull_list;
    WEngineListbox    engine_list;
    WBeamListbox      beam_list;
    WLauncherListbox  torp_list;

    WHullInfo         hull_info;
    WBeamInfo         beam_info;
    WTorpInfo         torp_info;
    WEngineInfo       engine_info;

    WPartStorageGroup hull_storage, torp_storage, beam_storage;
    WEngineStorageInfo engine_storage;
    WEngineInfoDiagram engine_diag;

    WPartCountGroup beam_count, torp_count;

 protected:
    GStarbaseBuildShipAction  action;

    void onPlanetChanged(int);

 private:
    WItemCostDisplay  item_cost_dpy;
    WOrderDisplay     order_dpy;

    UICardGroup       cards;
    UICardGroupTab    tabs;

    SignalConnection conn_update;

    // Must be declared exactly here so that it dies first.
 protected:
    ObjectHolder      h;

    void updateHullPage();
    void updateBeamPage();
    void updateEnginePage();
    void updateLauncherPage();
    void updateBuildOrder();
 public:
    WShipBuildScreen(string_t name, GPlanet& the_planet);
    ~WShipBuildScreen();

    bool handleCommand(const UICommand& cmd, UIBaseWidget& sender);
    bool handleEvent(const UIEvent& event, bool second_pass);
};

class WShipBuildScreenWithActor : public WShipBuildScreen {
    GPlanet& the_planet;

    void onOk();
    void onCancel();
    void onCancelBuild();
    void onToggleUseParts();
    void updateUseParts();
 public:
    WShipBuildScreenWithActor(GPlanet& the_planet);
    ~WShipBuildScreenWithActor();
};

#endif
