/**
  *  \file client/dlg-warp.cc
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/dlg-warp.h"
#include "game/ship.h"
#include "ui/skin.h"
#include "util/nls.h"
#include "util/misc.h"
#include "ui/layout.h"
#include "ui/msgbox.h"
#include "cpluslib/assert.h"
#include "gfx/palette.h"
#include "game/global.h"
#include "ui/widgets/multilinestatic.h"
#include "ui/widgets/framegroup.h"

static const int HYP_INDEX = 10;

static int
getIndexFromShip(GShip& s)
{
    // FIXME: if it HYPs, return 10
    return s.getWarp();
}

WShipSpeedSelector::WShipSpeedSelector()
    : UINumberSelector(0, 0, 9, 1), ship(0), orig_speed(0), turn_conn()
{ }

WShipSpeedSelector::WShipSpeedSelector(GShip& ship)
    : UINumberSelector(0, 0, 9, 1), ship(&ship), orig_speed(0), turn_conn(0)
{
    // FIXME: can we assert that? I hope, yes.
    ASSERT(&getDisplayedTurn().getCurrentUniverse().getShip(ship.getId()) == &ship);

    if (!ship.isPlayed()) {
        this->ship = 0;
    } else {
        /* we're playing this ship. */
        // FIXME: if it's not in a fleet, and can jump, offer HYP
        setValue(getIndexFromShip(ship));
        turn_conn = ship.sig_changed.add(this, &WShipSpeedSelector::shipUpdated);
        sig_changed.add(this, &WShipSpeedSelector::widgetUpdated);
        orig_speed = getValue();
    }
}

WShipSpeedSelector::~WShipSpeedSelector()
{ }

void
WShipSpeedSelector::shipUpdated(int)
{
    if (ship)
        setValue(getIndexFromShip(*ship));
}

void
WShipSpeedSelector::widgetUpdated(int)
{
    if (ship && getValue() != ship->getWarp()) {
        // FIXME: handle HYP
        ship->setWarp(getValue());
        getDisplayedTurn().getCurrentUniverse().doScreenUpdates();
    }
}

bool
WShipSpeedSelector::handleEvent(const UIEvent& e, bool second_pass)
{
    if (hasState(st_Focused) && e.type == ev_Key) {
        if (e.key.code >= '0' && e.key.code <= '9') {
            setValue(e.key.code - '0');
            return true;
        } else if (e.key.code == 'y' && getMax() >= HYP_INDEX) {
            setValue(HYP_INDEX);
            return true;
        }
    }
    return UINumberSelector::handleEvent(e, second_pass);
}

void
WShipSpeedSelector::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_TITLE]);
    ctx.setColor(UIColor::tc_Input);
    ctx.setSolidBackground();

    string_t st;
    if (getValue() == HYP_INDEX)
        st = _("Hyp");
    else
        st = itoa(getValue());

    outTextF(ctx, getExtent().x, getExtent().y, getExtent().h, st);

    can.update(getExtent());
}

void
WShipSpeedSelector::getLayoutInfo(LayoutInfo& li)
{
    li.flags = li_FlexibleH;
    li.min_size = li.pref_size = font_sizes[FONT_TITLE].scaledBy(3, 1);
}

void
WShipSpeedSelector::revert()
{
    setValue(orig_speed);
}

void
WShipSpeedSelector::doStandardDialog()
{
    UIWindow win(_("Set Speed"), UIWindow::BLUE_WINDOW, COLOR_GRAY);
    win.setLayout(UIVBoxLayout::instance5);

    UIMultilineStatic mls(FONT_NORMAL, _("Enter new warp factor and confirm with ENTER:"), 280);
    win.add(mls);

    UIGroup bn(UIHBoxLayout::instance5);
    UIFrameGroup fg(UIHBoxLayout::instance5);
    UIButton plus("+", '+'), minus("-", '-');
    fg.add(minus);
    fg.add(*this);
    fg.add(plus);

    bn.add(fg);

    UISpacer sp;
    UIButton ok(1, _("OK"), SDLK_RETURN);
    UIButton esc(0, _("Cancel"), SDLK_ESCAPE);
    ok.addStop();
    esc.addStop();
    bn.add(sp);
    bn.add(ok);
    bn.add(esc);
    win.add(bn);
    
    win.pack();

    setFocusedWidget();

    ui_root->moveWidgetToEdge(win, 0, 2, 10);
    ui_root->add(win);
    if (!ui_root->run())
        revert();
    ui_root->close(win);
}
