/**
  *  \file client/widgets/specbuildinfo.cc
  *  \brief Ship Specification Build Information
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/specbuildinfo.h"
#include "game/spec.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "ui/table.h"
#include "util/misc.h"
#include "game/host.h"
#include "game/team.h"


WSpecBuildInfo::WSpecBuildInfo()
    : hull(0)
{ }

WSpecBuildInfo::~WSpecBuildInfo()
{ }

void
WSpecBuildInfo::getLayoutInfo(LayoutInfo& info)
{
    info.pref_size = info.min_size = GfxPoint(300, font_heights[FONT_NORMAL]*5 + 5);
    info.flags = li_FlexibleH;
}

void
WSpecBuildInfo::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());

    if (hull > 0) {
        const GHull& h = ::getHull(hull);
        UITableWriter tw(ctx, GfxRect(getExtent().x, getExtent().y + font_heights[FONT_NORMAL], getExtent().w, getExtent().h - font_heights[FONT_NORMAL]), FONT_NORMAL);

        // Resources:
        tw.newLColumn();
        int x_res = tw.getCurrentPosition().x;
        tw.writeFixedCell(UIColor::tc_Static, _("Money:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Tritanium:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Duranium:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Molybdenum:"));

        tw.newRColumn(font_ems[FONT_NORMAL]*4);
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getCost().get(el_Money)));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getCost().get(el_Tritanium)));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getCost().get(el_Duranium)));
        tw.writeVariableCell(UIColor::tc_Green, numToString(h.getCost().get(el_Molybdenum)));

        tw.newLColumn();
        tw.writeFixedCell(UIColor::tc_Green, "mc");
        tw.writeFixedCell(UIColor::tc_Green, "kt");
        tw.writeFixedCell(UIColor::tc_Green, "kt");
        tw.writeFixedCell(UIColor::tc_Green, "kt");

        int x_pbp = 0;

        if (host.isPBPGame()) {
            // Leave a little room:
            tw.newRColumn(font_ems[FONT_NORMAL]);

            // Build Points:
            tw.newLColumn();
            x_pbp = tw.getCurrentPosition().x;

            int build, kill, scrap;
            int32_t mass = h.getMass();
            if (host.isPHost()) {
                /* Build */
                build = mass * config.PBPCostPer100KT(getPlayerId()) / 100;
                if (build < config.PBPMinimumCost(getPlayerId()))
                    build = config.PBPMinimumCost(getPlayerId());

                /* Kill, estimation (since there are many ways to destroy it) */
                kill = mass * (config.PALAggressorPointsPer10KT(getPlayerId()) + config.PALAggressorKillPointsPer10KT(getPlayerId())) / 10
                    + config.PALCombatAggressor(getPlayerId());

                /* Scrap */
                scrap = mass * config.PALRecyclingPer10KT(getPlayerId()) / 10;
            } else {
                /* Build:
                   - Vendetta (100 kt) => 2
                   - Loki (101 kt) => 3 */
                build = (mass + 49) / 50;

                /* Kill:
                   - Dwarfstar (100 kt) => 2 */
                kill = (mass / 100) + 1;

                /* Scrap */
                scrap = 1;
            }

            tw.writeFixedCell(UIColor::tc_Static, _("Build:"));
            tw.writeFixedCell(UIColor::tc_Static, _("Kill:"));
            tw.writeFixedCell(UIColor::tc_Static, _("Scrap:"));
            tw.newRColumn(font_ems[FONT_NORMAL]*4);
            tw.writeVariableCell(UIColor::tc_Green, numToString(build));
            tw.writeVariableCell(UIColor::tc_Green, numToString(kill));
            tw.writeVariableCell(UIColor::tc_Green, numToString(scrap));
        }
        tw.finish();

        // Headings
        ctx.useFont(*fonts[FONT_NORMAL]);
        ctx.setTextAlign(0, 0);
        ctx.setColor(UIColor::tc_Heading);
        ctx.setSolidBackground();
        outText(ctx, x_res, getExtent().y, _("Resources Needed:"));
        if (host.isPBPGame())
            outText(ctx, x_pbp, getExtent().y, _("Build Points:"));
    } else {
        drawBackground(ctx, getExtent());
    }
    can.update(getExtent());
}

void
WSpecBuildInfo::setHull(int hull)
{
    if (hull != this->hull) {
        this->hull = hull;
        drawWidget();
    }
}
