/**
  *  \file game/config.h
  *  \brief Game Configuration (HConfig/PConfig) Access
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CONFIG_H
#define PCC_V2_CONFIG_H

#include "cpluslib/string.h"
#include "cpluslib/types.h"
#include "io/io.h"
#include "io/streams.h"
#include "util/conf.h"
#include "game/struct.h"
#include "game/cost.h"
#include "game/playerset.h"

/** Option containing a Cost specification. */
class ConfigCostOption : public ConfigOption {
    GCost data[NUM_PLAYERS];
 public:
    ConfigCostOption(Config& container, const char* name);
    ~ConfigCostOption();
    void     set(string_t value);
    string_t toString() const;
    void     set(int index, const GCost& value);
    GCost operator()(int index) const;
};

/** Standard option type. Used for most options (arrayized!). */
typedef ConfigIntArrayOption<NUM_PLAYERS> ConfigStdOption;

/** Experience modificator option. */
typedef ConfigIntArrayOption<10> ConfigExpOption;

/** Game configuration. */
class GGameConfig : public Config {
 public:
    ConfigIntOption           ConfigLevel;
    ConfigStdOption           PlayerRace;
    ConfigStdOption           RecycleRate;
    ConfigIntOption           RandomMeteorRate;
    ConfigIntOption           AllowMinefields;
    ConfigIntOption           AllowAlchemy;
    ConfigIntOption           DeleteOldMessages;
    ConfigIntOption           DisablePasswords;
    ConfigStdOption           GroundKillFactor;
    ConfigStdOption           GroundDefenseFactor;
    ConfigStdOption           FreeFighters;
    ConfigStdOption           RaceMiningRate;
    ConfigStdOption           AllowBuildFighters;
    ConfigIntOption           RebelsBuildFighters;
    ConfigIntOption           ColoniesBuildFighters;
    ConfigIntOption           RobotsBuildFighters;
    ConfigStdOption           CloakFailureRate;
    ConfigIntOption           RobCloakedShips;
    ConfigStdOption           ScanRange;
    ConfigStdOption           SensorRange;
    ConfigStdOption           DarkSenseRange;
    ConfigIntOption           AllowHiss;
    ConfigIntOption           AllowRebelGroundAttack;
    ConfigIntOption           AllowSuperRefit;
    ConfigIntOption           AllowWebMines;
    ConfigStdOption           CloakFuelBurn;
    ConfigIntOption           AllowNewNatives;
    ConfigIntOption           AllowPlanetAttacks;
    ConfigStdOption           BorgAssimilationRate;
    ConfigStdOption           MineDecayRate;
    ConfigStdOption           WebMineDecayRate;
    ConfigStdOption           MaximumMinefieldRadius;
    ConfigStdOption           MaximumWebMinefieldRadius;
    ConfigIntOption           TransuraniumDecayRate;
    ConfigStdOption           StructureDecayPerTurn;
    ConfigIntOption           StructureDecayOnUnowned;
    ConfigIntOption           ClimateLimitsPopulation;
    ConfigStdOption           AllowEatingSupplies;
    ConfigIntOption           AllowNoFuelMovement;
    ConfigStdOption           MineHitOdds;
    ConfigStdOption           WebMineHitOdds;
    ConfigStdOption           MineScanRange;
    ConfigIntOption           AllowMinesDestroyMines;
    ConfigIntOption           AllowEngineShieldBonus;
    ConfigStdOption           EngineShieldBonusRate;
    ConfigStdOption           FighterSweepRate;
    ConfigIntOption           AllowColoniesSweepWebs;
    ConfigStdOption           MineSweepRate;
    ConfigStdOption           WebMineSweepRate;
    ConfigStdOption           HissEffectRate;
    ConfigStdOption           RobFailureOdds;
    ConfigIntOption           PlanetsAttackRebels;
    ConfigIntOption           PlanetsAttackKlingons;
    ConfigStdOption           MineSweepRange;
    ConfigStdOption           WebMineSweepRange;
    ConfigIntOption           AllowScienceMissions;
    ConfigStdOption           MineHitOddsWhenCloakedX10;
    ConfigIntOption           DamageLevelForCloakFail;
    ConfigIntOption           AllowFedCombatBonus;
    ConfigIntOption           MeteorShowerOdds;
    ConfigIntArrayOption<8>   MeteorShowerOreRanges;
    ConfigIntOption           LargeMeteorsImpacting;
    ConfigIntArrayOption<8>   LargeMeteorOreRanges;
    ConfigIntOption           AllowMeteorMessages;
    ConfigIntOption           AllowOneEngineTowing;
    ConfigIntOption           AllowHyperWarps;
    ConfigStdOption           ClimateDeathRate;
    ConfigIntOption           AllowGravityWells;
    ConfigIntOption           CrystalsPreferDeserts;
    ConfigIntOption           AllowMinesDestroyWebs;
    ConfigStdOption           MaxPlanetaryIncome;
    ConfigIntOption           IonStormActivity;
    ConfigIntOption           AllowChunneling;
    ConfigIntOption           AllowDeluxeSuperSpy;
    ConfigIntOption           IonStormsHideMines;
    ConfigIntOption           AllowGloryDevice;
    ConfigIntOption           AllowAntiCloakShips;
    ConfigIntOption           AllowGamblingShips;
    ConfigIntOption           AllowCloakedShipsAttack;
    ConfigIntOption           AllowShipCloning;
    ConfigIntOption           AllowBoardingParties;
    ConfigIntOption           AllowImperialAssault;
    ConfigIntOption           RamScoopFuelPerLY;
    ConfigIntOption           AllowAdvancedRefinery;
    ConfigIntOption           AllowBioscanners;
    ConfigIntOption           HullTechNotSlowedByMines;
    ConfigIntOption           UseAccurateFuelModel;
    ConfigIntOption           DefenseForUndetectable;
    ConfigIntOption           DefenseToBlockBioscan;
    ConfigIntOption           FactoriesForDetectable;
    ConfigIntOption           MinesForDetectable;
    ConfigStdOption           FighterSweepRange;
    ConfigIntOption           MineHitDamageFor100KT;
    ConfigIntOption           WebHitDamageFor100KT;
    ConfigIntOption           AllowRegisteredFunctions;
    ConfigIntOption           GravityWellRange;
    ConfigStringOption        Language;
    ConfigIntOption           AllowPlayerMessages;
    ConfigStringOption        ScoringMethod;
    ConfigIntOption           TowedShipsBreakFree;
    ConfigIntOption           NativeClimateDeathRate;
    ConfigStdOption           AllowMoreThan50Targets;
    ConfigIntOption           CrystalSinTempBehavior;
    ConfigIntOption           RGANeedsBeams;
    ConfigIntOption           AllowRGAOnUnowned;
    ConfigIntOption           CPEnableLanguage;
    ConfigIntOption           CPEnableBigTargets;
    ConfigIntOption           CPEnableRaceName;
    ConfigIntOption           CPEnableAllies;
    ConfigIntOption           CPEnableMessage;
    ConfigIntOption           AllowAnonymousMessages;
    ConfigIntOption           DelayAllianceCommands;
    ConfigStdOption           TerraformRate;
    ConfigIntOption           MaxColTempSlope;
    ConfigIntOption           WebDrainFuelLoss;
    ConfigIntOption           WebHitFuelLoss;
    ConfigIntOption           AllowWormholes;
    ConfigIntOption           WrmDisplacement;
    ConfigIntOption           WrmRandDisplacement;
    ConfigIntOption           WrmStabilityAddX10;
    ConfigIntOption           WrmRandStability;
    ConfigIntOption           WrmMassAdd;
    ConfigIntOption           WrmRandMass;
    ConfigIntOption           WrmVoluntaryTravel;
    ConfigIntOption           WrmTravelDistDivisor;
    ConfigIntOption           WrmTravelWarpSpeed;
    ConfigIntOption           WrmTravelCloaked;
    ConfigIntOption           WrmEntryPowerX100;
    ConfigIntOption           CPEnableGive;
    ConfigIntOption           AllowTowCloakedShips;
    ConfigIntOption           RobCloakedChance;
    ConfigStdOption           UnitsPerTorpRate;
    ConfigStdOption           UnitsPerWebRate;
    ConfigIntOption           AllowESBonusAgainstPlanets;
    ConfigStdOption           ShipCloneCostRate;
    ConfigIntOption           AllowHyperjumpGravWells;
    ConfigIntOption           NativeCombatSurvivalRate;
    ConfigIntOption           AllowPrivateerTowCapture;
    ConfigIntOption           AllowCrystalTowCapture;
    ConfigIntOption           RoundGravityWells;
    ConfigIntOption           CPEnableSend;
    ConfigIntOption           CumulativePillaging;
    ConfigIntOption           AllowInterceptAttack;
    ConfigStdOption           RaceGrowthRate;
    ConfigStdOption           ProductionRate;
    ConfigStdOption           MineOddsWarpBonusX100;
    ConfigStdOption           CloakMineOddsWarpBonusX100;
    ConfigStdOption           WebMineOddsWarpBonusX100;
    ConfigStdOption           MineTravelSafeWarp;
    ConfigStdOption           CloakedMineTravelSafeWarp;
    ConfigStdOption           WebMineTravelSafeWarp;
    ConfigIntOption           AllowCloakFailMessages;
    ConfigIntOption           TonsScoreCountsPlanets;
    ConfigIntOption           AllowExtendedMissions;
    ConfigIntOption           ExtMissionsStartAt;
    ConfigIntOption           WormholeUFOsStartAt;
    ConfigIntOption           MaxShipsHissing;
    ConfigIntOption           SpyDetectionChance;
    ConfigIntOption           MapTruehullByPlayerRace;
    ConfigIntOption           AllowWraparoundMap;
    ConfigIntArrayOption<4>   WraparoundRectangle;
    ConfigIntOption           CPEnableRemote;
    ConfigIntOption           AllowAlliedChunneling;
    ConfigStdOption           ColonistTaxRate;
    ConfigStdOption           NativeTaxRate;
    ConfigIntOption           AllowAlternativeTowing;
    ConfigIntOption           AllowBeamUpClans;
    ConfigIntOption           AllowBeamUpMultiple;
    ConfigIntOption           DamageLevelForTerraformFail;
    ConfigIntOption           DamageLevelForAntiCloakFail;
    ConfigIntOption           DamageLevelForChunnelFail;
    ConfigIntOption           DamageLevelForHyperjumpFail;
    ConfigIntOption           MinimumChunnelDistance;
    ConfigStdOption           TowStrengthEngineScale;
    ConfigStdOption           TowStrengthDistanceScale;
    ConfigIntOption           AllowPriorityBuild;
    ConfigStdOption           SBQBuildPALBoost;
    ConfigStdOption           SBQNewBuildPALBoost;
    ConfigStdOption           SBQPointsForAging;
    ConfigStdOption           SBQBuildChangePenalty;
    ConfigStdOption           SBQBoostExpX100;
    ConfigStdOption           BuildChangeRelativePenalty;
    ConfigStdOption           PALDecayPerTurn;
    ConfigStdOption           PALPlayerRate;
    ConfigStdOption           PALCombatAggressor;
    ConfigStdOption           PALAggressorPointsPer10KT;
    ConfigStdOption           PALOpponentPointsPer10KT;
    ConfigStdOption           PALAggressorKillPointsPer10KT;
    ConfigStdOption           PALOpponentKillPointsPer10KT;
    ConfigStdOption           PALCombatPlanetScaling;
    ConfigStdOption           PALCombatBaseScaling;
    ConfigStdOption           PALShipCapturePer10Crew;
    ConfigStdOption           PALRecyclingPer10KT;
    ConfigStdOption           PALBoardingPartyPer10Crew;
    ConfigStdOption           PALGroundAttackPer100Clans;
    ConfigStdOption           PALGloryDevice;
    ConfigStdOption           PALGloryDevicePer10KT;
    ConfigStdOption           PALGloryDamagePer10KT;
    ConfigStdOption           PALGloryKillPer10KT;
    ConfigStdOption           PALImperialAssault;
    ConfigStdOption           PALRGA;
    ConfigStdOption           PALPillage;
    ConfigStdOption           PALIncludesESB;
    ConfigStdOption           FilterPlayerMessages;
    ConfigIntOption           AlternativeAntiCloak;
    ConfigStdOption           AntiCloakImmunity;
    ConfigStdOption           AllowMoreThan500Minefields;
    ConfigIntOption           NumMinefields;
    ConfigStdOption           PALShipMinekillPer10KT;
    ConfigStdOption           MaximumMinefieldsPerPlayer;
    ConfigIntOption           MineIdNeedsPermission;
    ConfigStringOption        BuildQueue;
    ConfigStdOption           PBPCostPer100KT;
    ConfigStdOption           PBPMinimumCost;
    ConfigStdOption           PBPCloneCostRate;
    ConfigIntOption           AllowShipNames;
    ConfigIntOption           BuildPointReport;
    ConfigIntOption           AlternativeMinesDestroyMines;
    ConfigIntOption           NumShips;
    ConfigIntOption           ExtendedSensorSweep;
    ConfigIntOption           ColonistCombatSurvivalRate;
    ConfigIntOption           NewNativesPerTurn;
    ConfigIntArrayOption<2>   NewNativesPopulationRange;
    ConfigIntArrayOption<9>   NewNativesRaceRate;
    ConfigIntArrayOption<9>   NewNativesGovernmentRate;
    ConfigStdOption           PlayerSpecialMission;
    ConfigIntOption           TowedShipsCooperate;
    ConfigStdOption           WrmScanRange;
    ConfigStdOption           FuelUsagePerFightFor100KT;
    ConfigStdOption           FuelUsagePerTurnFor100KT;
    ConfigIntOption           CPEnableEnemies;
    ConfigIntOption           CPEnableShow;
    ConfigIntOption           CPEnableRefit;
    ConfigIntOption           AllowIncompatibleConfiguration;
    ConfigCostOption          FreeFighterCost;
    ConfigCostOption          StarbaseCost;
    ConfigCostOption          BaseFighterCost;
    ConfigCostOption          ShipFighterCost;
    ConfigStdOption           MaximumFightersOnBase;
    ConfigStdOption           MaximumDefenseOnBase;
    ConfigIntOption           NumExperienceLevels;
    ConfigExpOption           ExperienceLevels;
    ConfigStringOption        ExperienceLevelNames;
    ConfigIntOption           ExperienceLimit;
    ConfigStdOption           EPRecrewScaling;
    ConfigIntOption           EPShipAging;
    ConfigIntOption           EPPlanetAging;
    ConfigIntOption           EPPlanetGovernment;
    ConfigIntOption           EPShipMovement100LY;
    ConfigIntOption           EPShipHyperjump;
    ConfigIntOption           EPShipChunnel;
    ConfigIntOption           EPShipIonStorm100MEV;
    ConfigIntOption           EPCombatKillScaling;
    ConfigIntOption           EPCombatDamageScaling;
    ConfigIntOption           EPShipAlchemy100KT;
    ConfigExpOption           EPCombatBoostRate;
    ConfigExpOption           EPCombatBoostLevel;
    ConfigStdOption           EPTrainingScale;
    ConfigIntOption           ExactExperienceReports;
    ConfigExpOption           EModBayRechargeRate;
    ConfigExpOption           EModBayRechargeBonus;
    ConfigExpOption           EModBeamRechargeRate;
    ConfigExpOption           EModBeamRechargeBonus;
    ConfigExpOption           EModTubeRechargeRate;
    ConfigExpOption           EModBeamHitFighterCharge;
    ConfigExpOption           EModTorpHitOdds;
    ConfigExpOption           EModBeamHitOdds;
    ConfigExpOption           EModBeamHitBonus;
    ConfigExpOption           EModStrikesPerFighter;
    ConfigExpOption           EModFighterBeamExplosive;
    ConfigExpOption           EModFighterBeamKill;
    ConfigExpOption           EModFighterMovementSpeed;
    ConfigExpOption           EModMaxFightersLaunched;
    ConfigExpOption           EModTorpHitBonus;
    ConfigExpOption           EModTubeRechargeBonus;
    ConfigExpOption           EModExtraFighterBays;
    ConfigExpOption           EModEngineShieldBonusRate;
    ConfigExpOption           EModShieldDamageScaling;
    ConfigExpOption           EModShieldKillScaling;
    ConfigExpOption           EModHullDamageScaling;
    ConfigExpOption           EModCrewKillScaling;
    ConfigExpOption           EModPlanetaryTorpsPerTube;
    ConfigExpOption           EModMineHitOddsBonus;
    ConfigStringOption        GameName;
    ConfigStdOption           BayRechargeRate;
    ConfigStdOption           BayRechargeBonus;
    ConfigStdOption           BeamRechargeRate;
    ConfigStdOption           BeamRechargeBonus;
    ConfigStdOption           TubeRechargeRate;
    ConfigStdOption           BeamHitFighterCharge;
    ConfigStdOption           BeamHitShipCharge;
    ConfigStdOption           TorpFiringRange;
    ConfigStdOption           BeamFiringRange;
    ConfigStdOption           TorpHitOdds;
    ConfigStdOption           BeamHitOdds;
    ConfigStdOption           BeamHitBonus;
    ConfigStdOption           StrikesPerFighter;
    ConfigStdOption           FighterKillOdds;
    ConfigStdOption           FighterBeamExplosive;
    ConfigStdOption           FighterBeamKill;
    ConfigStdOption           ShipMovementSpeed;
    ConfigStdOption           FighterMovementSpeed;
    ConfigStdOption           BayLaunchInterval;
    ConfigStdOption           MaxFightersLaunched;
    ConfigIntOption           AllowAlternativeCombat;
    ConfigIntOption           StandoffDistance;
    ConfigIntOption           PlanetsHaveTubes;
    ConfigStdOption           PlanetaryTorpsPerTube;
    ConfigIntOption           FireOnAttackFighters;
    ConfigStdOption           TorpHitBonus;
    ConfigStdOption           TubeRechargeBonus;
    ConfigStdOption           ShieldDamageScaling;
    ConfigStdOption           HullDamageScaling;
    ConfigStdOption           CrewKillScaling;
    ConfigStdOption           ShieldKillScaling;
    ConfigStdOption           ExtraFighterBays;
    ConfigStdOption           BeamHitFighterRange;
    ConfigStdOption           FighterFiringRange;
    ConfigIntOption           AllowVPAFeatures;
    ConfigAlias               CPEnableRumor;
    ConfigAlias               RaceTaxRate;
    ConfigAlias               CPNumMinefields;
    ConfigAlias               NativeClansRange;
    ConfigAlias               NativeTypeFrequencies;
    ConfigAlias               NativeGovFrequencies;
    ConfigIntOption           MinimumHappiness;
    ConfigStdOption           ColonistCombatCaptureRate;
    ConfigStdOption           EPAcademyScale;
    ConfigStdOption           UseBaseTorpsInCombat;

    GGameConfig();
    ~GGameConfig();

    void assignDefaults();
    void assignDependantOptions();
    void assignFromHConfigImage(const char data[], std::size_t size);
};

extern GGameConfig config;

void initConfig();

inline int raceId(int player) {
    if (player > 0 && player <= 11)
        return config.PlayerRace(player);
    else
        return player;
};

inline int raceMissionId(int player) {
    if (player > 0 && player <= 11)
        return config.PlayerSpecialMission(player);
    else
        return player;
};

string_t getExperienceLevelName(int nr);
int      getExperienceBonus(const ConfigExpOption& param, int level);

GPlayerSet getPlayersOfRace(int nr);
GPlayerSet getPlayersWhereEnabled(const ConfigStdOption& opt);
GPlayerSet getPlayersWhere(const ConfigStdOption& opt, int value);

extern const struct ModuleInitializer mod_config[];

#endif
