#!/usr/bin/perl -w
use strict;

my $opt_diff = grep /--diff/, @ARGV;

# Read names of all subroutines
my %subrMap;
open FILE, "< tests/test_opt.qs" or die;
while (<FILE>) {
    if (/^\s*sub\s+([a-z0-9.\$_]+)/i) {
        my $r = uc($1);
        die "Duplicate routine '$r'"
          if exists $subrMap{$r};
        $subrMap{$r} = 1
    }
}
close FILE;

# Check it and generate todo list
my @todo;
foreach (sort keys %subrMap) {
    if (/^IN\.(.*)/) {
        die "Missing 'OUT.$1'" if !exists $subrMap{"OUT.$1"};
        push @todo, $1
    } elsif (/^OUT\.(.*)/) {
        die "Missing 'IN.$1'" if !exists $subrMap{"IN.$1"};
    } elsif ($_ ne 'MAIN') {
        die "Misnamed '$_'"
    }
}

# Assemble it
if (system("perl scripts/c2asm.pl tests/test_opt.qs") != 0) {
    die "Assembler failure.";
}

# Check it
my $ok = 0;
my $fail = 0;
foreach (@todo) {
    # print "$_...\n";
    my $in  = `(echo .load tests/test_opt.qc; echo '.opt in.$_') | ./teststmt`;
    my $out = `(echo .load tests/test_opt.qc; echo '?out.$_') | ./teststmt`;
    if ($in ne $out) {
        print "========== Error in test case $_ ==========\n";
        if ($opt_diff) {
            open A, "> __a.tmp" or die;
            print A $in;
            close A;
            open B, "> __b.tmp" or die;
            print B $out;
            close B;
            system "diff -u -L Result __a.tmp -L Expected __b.tmp";
            unlink "__a.tmp";
            unlink "__b.tmp";
        } else {
            print "----- Optimizer result -------\n";
            print "$in\n";
            print "----- Expected result -------\n";
            print "$out\n";
        }
        ++$fail;
    } else {
        ++$ok;
    }
}
print "$ok tests ok, $fail tests failed\n";
exit ($fail != 0);
